# -*- coding: utf-8 -*-
# Copyright (c) 2019, Frappe Technologies Pvt. Ltd. and contributors
# For license information, please see license.txt

from __future__ import unicode_literals
import frappe, erpnext
from frappe import _
from frappe.model.document import Document
from frappe.utils import nowdate, getdate, add_days, flt
from erpnext.controllers.accounts_controller import AccountsController
from erpnext.accounts.general_ledger import make_gl_entries
from erpnext.loan_management.doctype.process_loan_interest_accrual.process_loan_interest_accrual import process_loan_interest_accrual_for_demand_loans

class LoanDisbursement(AccountsController):

	def validate(self):
		self.set_missing_values()

	def before_submit(self):
		self.set_status_and_amounts()

	def before_cancel(self):
		self.set_status_and_amounts(cancel=1)

	def on_submit(self):
		self.make_gl_entries()

	def on_cancel(self):
		self.make_gl_entries(cancel=1)

	def set_missing_values(self):
		if not self.disbursement_date:
			self.disbursement_date = nowdate()

		if not self.cost_center:
			self.cost_center = erpnext.get_default_cost_center(self.company)

		if not self.posting_date:
			self.posting_date = self.disbursement_date or nowdate()

		if not self.bank_account and self.applicant_type == "Customer":
			self.bank_account = frappe.db.get_value("Customer", self.applicant, "default_bank_account")

	def set_status_and_amounts(self, cancel=0):

		loan_details = frappe.get_all("Loan",
			fields = ["loan_amount", "disbursed_amount", "total_principal_paid", "status", "is_term_loan"],
			filters= { "name": self.against_loan }
		)[0]

		if loan_details.status == "Disbursed" and not loan_details.is_term_loan:
			process_loan_interest_accrual_for_demand_loans(posting_date=add_days(self.disbursement_date, -1),
				loan=self.against_loan)

		if cancel:
			disbursed_amount = loan_details.disbursed_amount - self.disbursed_amount
			if disbursed_amount == 0:
				status = "Sanctioned"
			elif disbursed_amount >= loan_details.disbursed_amount:
				status = "Disbursed"
			else:
				status = "Partially Disbursed"
		else:
			disbursed_amount = self.disbursed_amount + loan_details.disbursed_amount

			if flt(disbursed_amount) - flt(loan_details.total_principal_paid) > flt(loan_details.loan_amount):
				frappe.throw(_("Disbursed Amount cannot be greater than loan amount"))

			if flt(disbursed_amount) >= loan_details.disbursed_amount:
				status = "Disbursed"
			else:
				status = "Partially Disbursed"

		frappe.db.set_value("Loan", self.against_loan, {
			"disbursement_date": self.disbursement_date,
			"disbursed_amount": disbursed_amount,
			"status": status
		})

	def make_gl_entries(self, cancel=0, adv_adj=0):
		gle_map = []
		loan_details = frappe.get_doc("Loan", self.against_loan)

		gle_map.append(
			self.get_gl_dict({
				"account": loan_details.loan_account,
				"against": loan_details.applicant,
				"debit": self.disbursed_amount,
				"debit_in_account_currency": self.disbursed_amount,
				"against_voucher_type": "Loan",
				"against_voucher": self.against_loan,
				"remarks": "Against Loan:" + self.against_loan,
				"cost_center": self.cost_center,
				"party_type": self.applicant_type,
				"party": self.applicant,
				"posting_date": self.disbursement_date
			})
		)

		gle_map.append(
			self.get_gl_dict({
				"account": loan_details.payment_account,
				"against": loan_details.applicant,
				"credit": self.disbursed_amount,
				"credit_in_account_currency": self.disbursed_amount,
				"against_voucher_type": "Loan",
				"against_voucher": self.against_loan,
				"remarks": "Against Loan:" + self.against_loan,
				"cost_center": self.cost_center,
				"party_type": self.applicant_type,
				"party": self.applicant,
				"posting_date": self.disbursement_date
			})
		)

		if gle_map:
			make_gl_entries(gle_map, cancel=cancel, adv_adj=adv_adj)
